@@ -1,18 +1,28 @@
 /*
- * Copyright 2010-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license
- * that can be found in the license/LICENSE.txt file.
+ * Copyright 2010-2018 JetBrains s.r.o. and Kotlin Programming Language contributors.
+ * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
  */
 
 package kotlin.internal
 
+import java.lang.reflect.Method
 import java.util.regex.MatchResult
 import kotlin.random.FallbackThreadLocalRandom
 import kotlin.random.Random
 
 internal open class PlatformImplementations {
 
+    private object ReflectAddSuppressedMethod {
+        @JvmField
+        public val method: Method? = Throwable::class.java.let { throwableClass ->
+            throwableClass.methods.find {
+                it.name == "addSuppressed" && it.parameterTypes.singleOrNull() == throwableClass
+            }
+        }
+    }
+
     public open fun addSuppressed(cause: Throwable, exception: Throwable) {
-        // do nothing
+        ReflectAddSuppressedMethod.method?.invoke(cause, exception)
     }
 
     public open fun getMatchResultNamedGroup(matchResult: MatchResult, name: String): MatchGroup? {
