@@ -1,18 +1,14 @@
 /*
- * Copyright 2010-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license
- * that can be found in the license/LICENSE.txt file.
+ * Copyright 2010-2018 JetBrains s.r.o. and Kotlin Programming Language contributors.
+ * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
  */
 
 @file:kotlin.jvm.JvmMultifileClass
 @file:kotlin.jvm.JvmName("StringsKt")
 
-
 package kotlin.text
 
-import kotlin.*
-import kotlin.comparisons.*
-import kotlin.internal.contracts.*
-
+import kotlin.contracts.contract
 
 /**
  * Returns a sub sequence of this char sequence having leading and trailing characters matching the [predicate] removed.
@@ -217,6 +213,8 @@ public fun String.padEnd(length: Int, padChar: Char = ' '): String =
 
 /**
  * Returns `true` if this nullable char sequence is either `null` or empty.
+ *
+ * @sample samples.text.Strings.stringIsNullOrEmpty
  */
 @kotlin.internal.InlineOnly
 public inline fun CharSequence?.isNullOrEmpty(): Boolean {
@@ -229,12 +227,16 @@ public inline fun CharSequence?.isNullOrEmpty(): Boolean {
 
 /**
  * Returns `true` if this char sequence is empty (contains no characters).
+ *
+ * @sample samples.text.Strings.stringIsEmpty
  */
 @kotlin.internal.InlineOnly
 public inline fun CharSequence.isEmpty(): Boolean = length == 0
 
 /**
  * Returns `true` if this char sequence is not empty.
+ *
+ * @sample samples.text.Strings.stringIsNotEmpty
  */
 @kotlin.internal.InlineOnly
 public inline fun CharSequence.isNotEmpty(): Boolean = length > 0
@@ -245,12 +247,16 @@ public inline fun CharSequence.isNotEmpty(): Boolean = length > 0
 
 /**
  * Returns `true` if this char sequence is not empty and contains some characters except of whitespace characters.
+ *
+ * @sample samples.text.Strings.stringIsNotBlank
  */
 @kotlin.internal.InlineOnly
 public inline fun CharSequence.isNotBlank(): Boolean = !isBlank()
 
 /**
  * Returns `true` if this nullable char sequence is either `null` or empty or consists solely of whitespace characters.
+ *
+ * @sample samples.text.Strings.stringIsNullOrBlank
  */
 @kotlin.internal.InlineOnly
 public inline fun CharSequence?.isNullOrBlank(): Boolean {
@@ -276,6 +282,28 @@ public operator fun CharSequence.iterator(): CharIterator = object : CharIterato
 @kotlin.internal.InlineOnly
 public inline fun String?.orEmpty(): String = this ?: ""
 
+/**
+ * Returns this char sequence if it's not empty
+ * or the result of calling [defaultValue] function if the char sequence is empty.
+ *
+ * @sample samples.text.Strings.stringIfEmpty
+ */
+@SinceKotlin("1.3")
+@kotlin.internal.InlineOnly
+public inline fun <C, R> C.ifEmpty(defaultValue: () -> R): R where C : CharSequence, C : R =
+    if (isEmpty()) defaultValue() else this
+
+/**
+ * Returns this char sequence if it is not empty and doesn't consist solely of whitespace characters,
+ * or the result of calling [defaultValue] function otherwise.
+ *
+ * @sample samples.text.Strings.stringIfBlank
+ */
+@SinceKotlin("1.3")
+@kotlin.internal.InlineOnly
+public inline fun <C, R> C.ifBlank(defaultValue: () -> R): R where C : CharSequence, C : R =
+    if (isBlank()) defaultValue() else this
+
 /**
  * Returns the range of valid character indices for this char sequence.
  */
@@ -754,6 +782,7 @@ public fun CharSequence.endsWith(suffix: CharSequence, ignoreCase: Boolean = fal
  * If this and [other] have no common prefix, returns the empty string.
 
  * @param ignoreCase `true` to ignore character case when matching a character. By default `false`.
+ * @sample samples.text.Strings.commonPrefixWith
  */
 public fun CharSequence.commonPrefixWith(other: CharSequence, ignoreCase: Boolean = false): String {
     val shortestLength = minOf(this.length, other.length)
@@ -774,6 +803,7 @@ public fun CharSequence.commonPrefixWith(other: CharSequence, ignoreCase: Boolea
  * If this and [other] have no common suffix, returns the empty string.
 
  * @param ignoreCase `true` to ignore character case when matching a character. By default `false`.
+ * @sample samples.text.Strings.commonSuffixWith
  */
 public fun CharSequence.commonSuffixWith(other: CharSequence, ignoreCase: Boolean = false): String {
     val thisLength = this.length
@@ -1043,7 +1073,7 @@ private class DelimitedRangesSequence(
     private val input: CharSequence,
     private val startIndex: Int,
     private val limit: Int,
-    private val getNextMatch: CharSequence.(Int) -> Pair<Int, Int>?
+    private val getNextMatch: CharSequence.(currentIndex: Int) -> Pair<Int, Int>?
 ) : Sequence<IntRange> {
 
     override fun iterator(): Iterator<IntRange> = object : Iterator<IntRange> {
@@ -1110,8 +1140,8 @@ private class DelimitedRangesSequence(
 private fun CharSequence.rangesDelimitedBy(delimiters: CharArray, startIndex: Int = 0, ignoreCase: Boolean = false, limit: Int = 0): Sequence<IntRange> {
     require(limit >= 0, { "Limit must be non-negative, but was $limit." })
 
-    return DelimitedRangesSequence(this, startIndex, limit, { startIndex ->
-        indexOfAny(delimiters, startIndex, ignoreCase = ignoreCase).let { if (it < 0) null else it to 1 }
+    return DelimitedRangesSequence(this, startIndex, limit, { currentIndex ->
+        indexOfAny(delimiters, currentIndex, ignoreCase = ignoreCase).let { if (it < 0) null else it to 1 }
     })
 }
 
@@ -1134,7 +1164,7 @@ private fun CharSequence.rangesDelimitedBy(delimiters: Array<out String>, startI
     require(limit >= 0, { "Limit must be non-negative, but was $limit." } )
     val delimitersList = delimiters.asList()
 
-    return DelimitedRangesSequence(this, startIndex, limit, { startIndex -> findAnyOf(delimitersList, startIndex, ignoreCase = ignoreCase, last = false)?.let { it.first to it.second.length } })
+    return DelimitedRangesSequence(this, startIndex, limit, { currentIndex -> findAnyOf(delimitersList, currentIndex, ignoreCase = ignoreCase, last = false)?.let { it.first to it.second.length } })
 
 }
 
@@ -1244,10 +1274,14 @@ public inline fun CharSequence.split(regex: Regex, limit: Int = 0): List<String>
 
 /**
  * Splits this char sequence to a sequence of lines delimited by any of the following character sequences: CRLF, LF or CR.
+ *
+ * The lines returned do not include terminating line separators.
  */
 public fun CharSequence.lineSequence(): Sequence<String> = splitToSequence("\r\n", "\n", "\r")
 
 /**
- * * Splits this char sequence to a list of lines delimited by any of the following character sequences: CRLF, LF or CR.
+ * Splits this char sequence to a list of lines delimited by any of the following character sequences: CRLF, LF or CR.
+ *
+ * The lines returned do not include terminating line separators.
  */
 public fun CharSequence.lines(): List<String> = lineSequence().toList()
