@@ -20,13 +20,16 @@ import org.jetbrains.kotlin.descriptors.*
 import org.jetbrains.kotlin.descriptors.impl.DeclarationDescriptorVisitorEmptyBodies
 import org.jetbrains.kotlin.load.java.JvmAbi
 import org.jetbrains.kotlin.name.Name
+import org.jetbrains.kotlin.renderer.DescriptorRenderer
 import org.jetbrains.kotlin.resolve.scopes.MemberScope
 import java.lang.reflect.Constructor
 import java.lang.reflect.Method
 import kotlin.jvm.internal.ClassBasedDeclarationContainer
-import kotlin.reflect.jvm.internal.components.RuntimeModuleData
-import kotlin.reflect.jvm.internal.structure.createArrayType
-import kotlin.reflect.jvm.internal.structure.safeClassLoader
+import org.jetbrains.kotlin.descriptors.runtime.components.RuntimeModuleData
+import org.jetbrains.kotlin.descriptors.runtime.components.tryLoadClass
+import org.jetbrains.kotlin.descriptors.runtime.structure.createArrayType
+import org.jetbrains.kotlin.descriptors.runtime.structure.safeClassLoader
+import org.jetbrains.kotlin.descriptors.runtime.structure.wrapperByPrimitive
 
 internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContainer {
     abstract inner class Data {
@@ -37,7 +40,7 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
     }
 
     protected open val methodOwner: Class<*>
-        get() = jClass
+        get() = jClass.wrapperByPrimitive ?: jClass
 
     abstract val constructorDescriptors: Collection<ConstructorDescriptor>
 
@@ -50,21 +53,20 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
     protected fun getMembers(scope: MemberScope, belonginess: MemberBelonginess): Collection<KCallableImpl<*>> {
         val visitor = object : DeclarationDescriptorVisitorEmptyBodies<KCallableImpl<*>, Unit>() {
             override fun visitPropertyDescriptor(descriptor: PropertyDescriptor, data: Unit): KCallableImpl<*> =
-                    createProperty(descriptor)
+                createProperty(descriptor)
 
             override fun visitFunctionDescriptor(descriptor: FunctionDescriptor, data: Unit): KCallableImpl<*> =
-                    KFunctionImpl(this@KDeclarationContainerImpl, descriptor)
+                KFunctionImpl(this@KDeclarationContainerImpl, descriptor)
 
             override fun visitConstructorDescriptor(descriptor: ConstructorDescriptor, data: Unit): KCallableImpl<*> =
-                    throw IllegalStateException("No constructors should appear in this scope: $descriptor")
+                throw IllegalStateException("No constructors should appear in this scope: $descriptor")
         }
 
         return scope.getContributedDescriptors().mapNotNull { descriptor ->
             if (descriptor is CallableMemberDescriptor &&
                 descriptor.visibility != Visibilities.INVISIBLE_FAKE &&
-                belonginess.accept(descriptor))
-                descriptor.accept(visitor, Unit)
-            else null
+                belonginess.accept(descriptor)
+            ) descriptor.accept(visitor, Unit) else null
         }.toList()
     }
 
@@ -73,12 +75,12 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
         INHERITED;
 
         fun accept(member: CallableMemberDescriptor): Boolean =
-                member.kind.isReal == (this == DECLARED)
+            member.kind.isReal == (this == DECLARED)
     }
 
     private fun createProperty(descriptor: PropertyDescriptor): KPropertyImpl<*> {
         val receiverCount = (descriptor.dispatchReceiverParameter?.let { 1 } ?: 0) +
-                            (descriptor.extensionReceiverParameter?.let { 1 } ?: 0)
+                (descriptor.extensionReceiverParameter?.let { 1 } ?: 0)
 
         when {
             descriptor.isVar -> when (receiverCount) {
@@ -101,13 +103,13 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
         if (match != null) {
             val (number) = match.destructured
             return getLocalProperty(number.toInt())
-                   ?: throw KotlinReflectionInternalError("Local property #$number not found in $jClass")
+                    ?: throw KotlinReflectionInternalError("Local property #$number not found in $jClass")
         }
 
         val properties = getProperties(Name.identifier(name))
-                .filter { descriptor ->
-                    RuntimeTypeMapper.mapPropertySignature(descriptor).asString() == signature
-                }
+            .filter { descriptor ->
+                RuntimeTypeMapper.mapPropertySignature(descriptor).asString() == signature
+            }
 
         if (properties.isEmpty()) {
             throw KotlinReflectionInternalError("Property '$name' (JVM signature: $signature) not resolved in $this")
@@ -125,16 +127,20 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
             // TODO: consider writing additional info (besides signature) to property reference objects to distinguish them in this case
 
             val mostVisibleProperties = properties
-                    .groupBy { it.visibility }
-                    .toSortedMap(Comparator { first, second ->
-                        Visibilities.compare(first, second) ?: 0
-                    }).values.last()
+                .groupBy { it.visibility }
+                .toSortedMap(Comparator { first, second ->
+                    Visibilities.compare(first, second) ?: 0
+                }).values.last()
             if (mostVisibleProperties.size == 1) {
                 return mostVisibleProperties.first()
             }
 
+            val allMembers = getProperties(Name.identifier(name)).joinToString("\n") { descriptor ->
+                DescriptorRenderer.DEBUG_TEXT.render(descriptor) + " | " + RuntimeTypeMapper.mapPropertySignature(descriptor)
+            }
             throw KotlinReflectionInternalError(
-                    "${properties.size} properties '$name' (JVM signature: $signature) resolved in $this: $properties"
+                "Property '$name' (JVM signature: $signature) not resolved in $this:" +
+                        if (allMembers.isEmpty()) " no members found" else "\n$allMembers"
             )
         }
 
@@ -142,100 +148,118 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
     }
 
     fun findFunctionDescriptor(name: String, signature: String): FunctionDescriptor {
-        val functions = (if (name == "<init>") constructorDescriptors.toList() else getFunctions(Name.identifier(name)))
-                .filter { descriptor ->
-                    RuntimeTypeMapper.mapSignature(descriptor).asString() == signature
-                }
+        val members = if (name == "<init>") constructorDescriptors.toList() else getFunctions(Name.identifier(name))
+        val functions = members.filter { descriptor ->
+            RuntimeTypeMapper.mapSignature(descriptor).asString() == signature
+        }
 
         if (functions.size != 1) {
-            val debugText = "'$name' (JVM signature: $signature)"
+            val allMembers = members.joinToString("\n") { descriptor ->
+                DescriptorRenderer.DEBUG_TEXT.render(descriptor) + " | " + RuntimeTypeMapper.mapSignature(descriptor)
+            }
             throw KotlinReflectionInternalError(
-                    if (functions.isEmpty()) "Function $debugText not resolved in $this"
-                    else "${functions.size} functions $debugText resolved in $this: $functions"
+                "Function '$name' (JVM signature: $signature) not resolved in $this:" +
+                        if (allMembers.isEmpty()) " no members found" else "\n$allMembers"
             )
         }
 
         return functions.single()
     }
 
-    private fun Class<*>.lookupMethod(name: String, parameterTypes: List<Class<*>>, returnType: Class<*>, isPublic: Boolean): Method? {
-        val parametersArray = parameterTypes.toTypedArray()
-
-        // If we're looking for a public method, just use Java reflection's getMethod/getMethods
-        if (isPublic) {
-            return tryGetMethod(name, parametersArray, returnType, declared = false)
+    private fun Class<*>.lookupMethod(
+        name: String, parameterTypes: Array<Class<*>>, returnType: Class<*>, isStaticDefault: Boolean
+    ): Method? {
+        // Static "$default" method in any class takes an instance of that class as the first parameter.
+        if (isStaticDefault) {
+            parameterTypes[0] = this
         }
 
-        // If we're looking for a non-public method, it might be located not only in this class, but also in any of its superclasses
-        var klass: Class<*>? = this
-        while (klass != null) {
-            val method = klass.tryGetMethod(name, parametersArray, returnType, declared = true)
-            if (method != null) return method
-            klass = klass.superclass
+        tryGetMethod(name, parameterTypes, returnType)?.let { return it }
+
+        superclass?.lookupMethod(name, parameterTypes, returnType, isStaticDefault)?.let { return it }
+
+        // TODO: avoid exponential complexity here
+        for (superInterface in interfaces) {
+            superInterface.lookupMethod(name, parameterTypes, returnType, isStaticDefault)?.let { return it }
+
+            // Static "$default" methods should be looked up in each DefaultImpls class, see KT-33430
+            if (isStaticDefault) {
+                val defaultImpls = superInterface.classLoader.tryLoadClass(superInterface.name + JvmAbi.DEFAULT_IMPLS_SUFFIX)
+                if (defaultImpls != null) {
+                    parameterTypes[0] = superInterface
+                    defaultImpls.tryGetMethod(name, parameterTypes, returnType)?.let { return it }
+                }
+            }
         }
 
         return null
     }
 
-    private fun Class<*>.tryGetMethod(name: String, parameterTypes: Array<Class<*>>, returnType: Class<*>, declared: Boolean): Method? =
-            try {
-                val result = if (declared) getDeclaredMethod(name, *parameterTypes) else getMethod(name, *parameterTypes)
+    private fun Class<*>.tryGetMethod(name: String, parameterTypes: Array<Class<*>>, returnType: Class<*>): Method? =
+        try {
+            val result = getDeclaredMethod(name, *parameterTypes)
 
-                if (result.returnType == returnType) result
-                else {
-                    // If we've found a method with an unexpected return type, it's likely that there are several methods in this class
-                    // with the given parameter types and Java reflection API has returned not the one we're looking for.
-                    // Falling back to enumerating all methods in the class in this (rather rare) case.
-                    // Example: class A(val x: Int) { fun getX(): String = ... }
-                    val allMethods = if (declared) declaredMethods else methods
-                    allMethods.firstOrNull { method ->
-                        method.name == name &&
-                        method.returnType == returnType &&
-                        method.parameterTypes.contentEquals(parameterTypes)
-                    }
+            if (result.returnType == returnType) result
+            else {
+                // If we've found a method with an unexpected return type, it's likely that there are several methods in this class
+                // with the given parameter types and Java reflection API has returned not the one we're looking for.
+                // Falling back to enumerating all methods in the class in this (rather rare) case.
+                // Example: class A(val x: Int) { fun getX(): String = ... }
+                declaredMethods.firstOrNull { method ->
+                    method.name == name &&
+                            method.returnType == returnType &&
+                            method.parameterTypes!!.contentEquals(parameterTypes)
                 }
             }
-            catch (e: NoSuchMethodException) {
-                null
-            }
+        } catch (e: NoSuchMethodException) {
+            null
+        }
 
-    private fun Class<*>.tryGetConstructor(parameterTypes: List<Class<*>>, declared: Boolean): Constructor<*>? =
-            try {
-                if (declared) getDeclaredConstructor(*parameterTypes.toTypedArray())
-                else getConstructor(*parameterTypes.toTypedArray())
-            }
-            catch (e: NoSuchMethodException) {
-                null
-            }
+    private fun Class<*>.tryGetConstructor(parameterTypes: List<Class<*>>): Constructor<*>? =
+        try {
+            getDeclaredConstructor(*parameterTypes.toTypedArray())
+        } catch (e: NoSuchMethodException) {
+            null
+        }
 
-    fun findMethodBySignature(name: String, desc: String, isPublic: Boolean): Method? {
+    fun findMethodBySignature(name: String, desc: String): Method? {
         if (name == "<init>") return null
 
-        return methodOwner.lookupMethod(name, loadParameterTypes(desc), loadReturnType(desc), isPublic)
+        val parameterTypes = loadParameterTypes(desc).toTypedArray()
+        val returnType = loadReturnType(desc)
+        methodOwner.lookupMethod(name, parameterTypes, returnType, false)?.let { return it }
+
+        // Methods from java.lang.Object (equals, hashCode, toString) cannot be found in the interface via
+        // Class.getMethod/getDeclaredMethod, so for interfaces, we also look in java.lang.Object.
+        if (methodOwner.isInterface) {
+            Any::class.java.lookupMethod(name, parameterTypes, returnType, false)?.let { return it }
+        }
+
+        return null
     }
 
-    fun findDefaultMethod(name: String, desc: String, isMember: Boolean, isPublic: Boolean): Method? {
+    fun findDefaultMethod(name: String, desc: String, isMember: Boolean): Method? {
         if (name == "<init>") return null
 
         val parameterTypes = arrayListOf<Class<*>>()
         if (isMember) {
+            // Note that this value is replaced inside the lookupMethod call below, for each class/interface in the hierarchy.
             parameterTypes.add(jClass)
         }
         addParametersAndMasks(parameterTypes, desc, false)
 
-        return methodOwner.lookupMethod(name + JvmAbi.DEFAULT_PARAMS_IMPL_SUFFIX, parameterTypes, loadReturnType(desc), isPublic)
+        return methodOwner.lookupMethod(
+            name + JvmAbi.DEFAULT_PARAMS_IMPL_SUFFIX, parameterTypes.toTypedArray(), loadReturnType(desc), isStaticDefault = isMember
+        )
     }
 
-    fun findConstructorBySignature(desc: String, isPublic: Boolean): Constructor<*>? {
-        return jClass.tryGetConstructor(loadParameterTypes(desc), declared = !isPublic)
-    }
+    fun findConstructorBySignature(desc: String): Constructor<*>? =
+        jClass.tryGetConstructor(loadParameterTypes(desc))
 
-    fun findDefaultConstructor(desc: String, isPublic: Boolean): Constructor<*>? {
-        val parameterTypes = arrayListOf<Class<*>>()
-        addParametersAndMasks(parameterTypes, desc, true)
-
-        return jClass.tryGetConstructor(parameterTypes, declared = !isPublic)
-    }
+    fun findDefaultConstructor(desc: String): Constructor<*>? =
+        jClass.tryGetConstructor(arrayListOf<Class<*>>().also { parameterTypes ->
+            addParametersAndMasks(parameterTypes, desc, true)
+        })
 
     private fun addParametersAndMasks(result: MutableList<Class<*>>, desc: String, isConstructor: Boolean) {
         val valueParameters = loadParameterTypes(desc)
@@ -267,23 +291,23 @@ internal abstract class KDeclarationContainerImpl : ClassBasedDeclarationContain
     }
 
     private fun parseType(desc: String, begin: Int, end: Int): Class<*> =
-            when (desc[begin]) {
-                'L' -> jClass.safeClassLoader.loadClass(desc.substring(begin + 1, end - 1).replace('/', '.'))
-                '[' -> parseType(desc, begin + 1, end).createArrayType()
-                'V' -> Void.TYPE
-                'Z' -> Boolean::class.java
-                'C' -> Char::class.java
-                'B' -> Byte::class.java
-                'S' -> Short::class.java
-                'I' -> Int::class.java
-                'F' -> Float::class.java
-                'J' -> Long::class.java
-                'D' -> Double::class.java
-                else -> throw KotlinReflectionInternalError("Unknown type prefix in the method signature: $desc")
-            }
+        when (desc[begin]) {
+            'L' -> jClass.safeClassLoader.loadClass(desc.substring(begin + 1, end - 1).replace('/', '.'))
+            '[' -> parseType(desc, begin + 1, end).createArrayType()
+            'V' -> Void.TYPE
+            'Z' -> Boolean::class.java
+            'C' -> Char::class.java
+            'B' -> Byte::class.java
+            'S' -> Short::class.java
+            'I' -> Int::class.java
+            'F' -> Float::class.java
+            'J' -> Long::class.java
+            'D' -> Double::class.java
+            else -> throw KotlinReflectionInternalError("Unknown type prefix in the method signature: $desc")
+        }
 
     private fun loadReturnType(desc: String): Class<*> =
-            parseType(desc, desc.indexOf(')') + 1, desc.length)
+        parseType(desc, desc.indexOf(')') + 1, desc.length)
 
     companion object {
         private val DEFAULT_CONSTRUCTOR_MARKER = Class.forName("kotlin.jvm.internal.DefaultConstructorMarker")
