@@ -18,6 +18,7 @@ import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.COROUTINES_PACKA
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.MemberScope
 import kotlin.reflect.jvm.internal.impl.storage.StorageManager
 import kotlin.reflect.jvm.internal.impl.types.*
+import kotlin.reflect.jvm.internal.impl.types.checker.KotlinTypeRefiner
 import java.util.*
 
 /**
@@ -82,7 +83,7 @@ class FunctionClassDescriptor(
 
     override fun getTypeConstructor(): TypeConstructor = typeConstructor
 
-    override fun getUnsubstitutedMemberScope() = memberScope
+    override fun getUnsubstitutedMemberScope(kotlinTypeRefiner: KotlinTypeRefiner) = memberScope
 
     override fun getCompanionObjectDescriptor() = null
     override fun getConstructors() = emptyList<ClassConstructorDescriptor>()
