@@ -24,30 +24,40 @@ import kotlin.reflect.jvm.internal.impl.descriptors.PackageViewDescriptor
 import kotlin.reflect.jvm.internal.impl.descriptors.annotations.Annotations
 import kotlin.reflect.jvm.internal.impl.name.FqName
 import kotlin.reflect.jvm.internal.impl.name.Name
-import kotlin.reflect.jvm.internal.impl.resolve.MultiTargetPlatform
+import kotlin.reflect.jvm.internal.impl.platform.TargetPlatform
 import kotlin.reflect.jvm.internal.impl.storage.StorageManager
+import kotlin.reflect.jvm.internal.impl.types.checker.REFINER_CAPABILITY
+import kotlin.reflect.jvm.internal.impl.types.checker.Ref
+import kotlin.reflect.jvm.internal.impl.types.refinement.TypeRefinement
 import kotlin.reflect.jvm.internal.impl.utils.sure
-import java.lang.IllegalArgumentException
 
 class ModuleDescriptorImpl @JvmOverloads constructor(
-        moduleName: Name,
-        private val storageManager: StorageManager,
-        override val builtIns: KotlinBuiltIns,
-        // May be null in compiler context, should be not-null in IDE context
-        multiTargetPlatform: MultiTargetPlatform? = null,
-        capabilities: Map<ModuleDescriptor.Capability<*>, Any?> = emptyMap()
+    moduleName: Name,
+    private val storageManager: StorageManager,
+    override val builtIns: KotlinBuiltIns,
+    // May be null in compiler context, should be not-null in IDE context
+    override val platform: TargetPlatform? = null,
+    capabilities: Map<ModuleDescriptor.Capability<*>, Any?> = emptyMap(),
+    override val stableName: Name? = null
 ) : DeclarationDescriptorImpl(Annotations.EMPTY, moduleName), ModuleDescriptor {
+    private val capabilities: Map<ModuleDescriptor.Capability<*>, Any?>
+
     init {
         if (!moduleName.isSpecial) {
             throw IllegalArgumentException("Module name must be special: $moduleName")
         }
+        this.capabilities = capabilities.toMutableMap()
+        @UseExperimental(TypeRefinement::class)
+        this.capabilities[REFINER_CAPABILITY] = Ref(null)
     }
 
-    private val capabilities = capabilities + (multiTargetPlatform?.let { mapOf(MultiTargetPlatform.CAPABILITY to it) } ?: emptyMap())
-
     private var dependencies: ModuleDependencies? = null
     private var packageFragmentProviderForModuleContent: PackageFragmentProvider? = null
 
+    val packageFragmentProviderForModuleContentWithoutDependencies: PackageFragmentProvider
+        get() = packageFragmentProviderForModuleContent
+            ?: throw IllegalStateException("Module $id was not initialized by the time it's content without dependencies was queried")
+
     override var isValid: Boolean = true
 
     override fun assertValid() {
@@ -56,12 +66,13 @@ class ModuleDescriptorImpl @JvmOverloads constructor(
         }
     }
 
-    private val packages = storageManager.createMemoizedFunction<FqName, PackageViewDescriptor> {
-        fqName: FqName -> LazyPackageViewDescriptorImpl(this, fqName, storageManager)
+    private val packages = storageManager.createMemoizedFunction<FqName, PackageViewDescriptor> { fqName: FqName ->
+        LazyPackageViewDescriptorImpl(this, fqName, storageManager)
     }
 
     @Deprecated("This method is not going to be supported. Please do not use it")
-    val testOnly_AllDependentModules: List<ModuleDescriptorImpl> get() = this.dependencies!!.allDependencies
+    val testOnly_AllDependentModules: List<ModuleDescriptorImpl>
+        get() = this.dependencies!!.allDependencies
 
     override val allDependencyModules: List<ModuleDescriptor>
         get() = this.dependencies.sure { "Dependencies of module $id were not set" }.allDependencies.filter { it != this }
@@ -83,8 +94,7 @@ class ModuleDescriptorImpl @JvmOverloads constructor(
         val moduleDependencies = dependencies.sure { "Dependencies of module $id were not set before querying module content" }
         val dependenciesDescriptors = moduleDependencies.allDependencies
         assert(this in dependenciesDescriptors) { "Module $id is not contained in his own dependencies, this is probably a misconfiguration" }
-        dependenciesDescriptors.forEach {
-            dependency ->
+        dependenciesDescriptors.forEach { dependency ->
             assert(dependency.isInitialized) {
                 "Dependency module ${dependency.id} was not initialized by the time contents of dependent module ${this.id} were queried"
             }
@@ -115,7 +125,12 @@ class ModuleDescriptorImpl @JvmOverloads constructor(
     }
 
     override fun shouldSeeInternalsOf(targetModule: ModuleDescriptor): Boolean {
-        return this == targetModule || targetModule in dependencies!!.modulesWhoseInternalsAreVisible || targetModule in expectedByModules
+        if (this == targetModule) return true
+        if (targetModule in dependencies!!.modulesWhoseInternalsAreVisible) return true
+        if (targetModule in expectedByModules) return true
+        if (this in targetModule.expectedByModules) return true
+
+        return false
     }
 
     private val id: String
@@ -135,14 +150,6 @@ class ModuleDescriptorImpl @JvmOverloads constructor(
             return packageFragmentProviderForWholeModuleWithDependencies
         }
 
-    val packageFragmentProviderForContent: PackageFragmentProvider
-        get() {
-            assertValid()
-            return packageFragmentProviderForModuleContent.sure {
-                "Module $id was not initialized by the time packageFragmentProviderForContent was requested"
-            }
-        }
-
     @Suppress("UNCHECKED_CAST")
     override fun <T> getCapability(capability: ModuleDescriptor.Capability<T>) = capabilities[capability] as? T
 }
