@@ -16,11 +16,15 @@
 
 package kotlin.reflect.jvm.internal.impl.load.java.components
 
+import kotlin.reflect.jvm.internal.impl.container.DefaultImplementation
+import kotlin.reflect.jvm.internal.impl.container.PlatformExtensionsClashResolver
+import kotlin.reflect.jvm.internal.impl.container.PlatformSpecificExtension
 import kotlin.reflect.jvm.internal.impl.load.java.descriptors.JavaClassDescriptor
 import kotlin.reflect.jvm.internal.impl.types.SimpleType
 
-interface SamConversionResolver {
-    companion object EMPTY : SamConversionResolver {
+@DefaultImplementation(impl = SamConversionResolver.Empty::class)
+interface SamConversionResolver : PlatformSpecificExtension<SamConversionResolver> {
+    object Empty : SamConversionResolver {
         override fun resolveFunctionTypeIfSamInterface(classDescriptor: JavaClassDescriptor): SimpleType? = null
     }
 
