@@ -1,6 +1,6 @@
 /*
- * Copyright 2000-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license
- * that can be found in the license/LICENSE.txt file.
+ * Copyright 2000-2018 JetBrains s.r.o. and Kotlin Programming Language contributors.
+ * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
  */
 
 package kotlin.reflect.jvm.internal.impl.load.java.lazy.descriptors
@@ -34,18 +34,21 @@ import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.resolveTopLevelCl
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.InnerClassesScopeWrapper
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.MemberScope
 import kotlin.reflect.jvm.internal.impl.types.*
+import kotlin.reflect.jvm.internal.impl.types.checker.KotlinTypeRefiner
 import kotlin.reflect.jvm.internal.impl.utils.addIfNotNull
 import kotlin.reflect.jvm.internal.impl.utils.addToStdlib.safeAs
 import java.util.*
 
 class LazyJavaClassDescriptor(
-        val outerContext: LazyJavaResolverContext,
-        containingDeclaration: DeclarationDescriptor,
-        val jClass: JavaClass,
-        private val additionalSupertypeClassDescriptor: ClassDescriptor? = null
-) : ClassDescriptorBase(outerContext.storageManager, containingDeclaration, jClass.name,
-                        outerContext.components.sourceElementFactory.source(jClass),
-                        /* isExternal = */ false), JavaClassDescriptor {
+    val outerContext: LazyJavaResolverContext,
+    containingDeclaration: DeclarationDescriptor,
+    val jClass: JavaClass,
+    private val additionalSupertypeClassDescriptor: ClassDescriptor? = null
+) : ClassDescriptorBase(
+    outerContext.storageManager, containingDeclaration, jClass.name,
+    outerContext.components.sourceElementFactory.source(jClass),
+    /* isExternal = */ false
+), JavaClassDescriptor {
 
     companion object {
         @JvmStatic
@@ -69,9 +72,9 @@ class LazyJavaClassDescriptor(
         else -> ClassKind.CLASS
     }
 
-    private val modality = if (jClass.isAnnotationType)
-                               Modality.FINAL
-                           else Modality.convertFromFlags(jClass.isAbstract || jClass.isInterface, !jClass.isFinal)
+    private val modality =
+        if (jClass.isAnnotationType || jClass.isEnum) Modality.FINAL
+        else Modality.convertFromFlags(jClass.isAbstract || jClass.isInterface, !jClass.isFinal)
 
     private val visibility = jClass.visibility
     private val isInner = jClass.outerClass != null && !jClass.isStatic
@@ -86,7 +89,7 @@ class LazyJavaClassDescriptor(
     // Kotlin considers this "private in package" just as "private" and thinks they are invisible for inheritors,
     // so their functions are invisible fake which is not true.
     override fun getVisibility() =
-            if (visibility == Visibilities.PRIVATE && jClass.outerClass == null) JavaVisibilities.PACKAGE_VISIBILITY else visibility
+        if (visibility == Visibilities.PRIVATE && jClass.outerClass == null) JavaVisibilities.PACKAGE_VISIBILITY else visibility
 
     override fun isInner() = isInner
     override fun isData() = false
@@ -98,10 +101,21 @@ class LazyJavaClassDescriptor(
     private val typeConstructor = LazyJavaClassTypeConstructor()
     override fun getTypeConstructor(): TypeConstructor = typeConstructor
 
-    private val unsubstitutedMemberScope = LazyJavaClassMemberScope(c, this, jClass)
-    override fun getUnsubstitutedMemberScope() = unsubstitutedMemberScope
+    private val unsubstitutedMemberScope =
+        LazyJavaClassMemberScope(c, this, jClass, skipRefinement = additionalSupertypeClassDescriptor != null)
 
-    private val innerClassesScope = InnerClassesScopeWrapper(getUnsubstitutedMemberScope())
+    private val scopeHolder =
+        ScopesHolderForClass.create(this, c.storageManager, c.components.kotlinTypeChecker.kotlinTypeRefiner) { kotlinTypeRefiner ->
+            LazyJavaClassMemberScope(
+                c, this, jClass,
+                skipRefinement = additionalSupertypeClassDescriptor != null,
+                mainScope = unsubstitutedMemberScope
+            )
+        }
+
+    override fun getUnsubstitutedMemberScope(kotlinTypeRefiner: KotlinTypeRefiner) = scopeHolder.getScope(kotlinTypeRefiner)
+
+    private val innerClassesScope = InnerClassesScopeWrapper(unsubstitutedMemberScope)
     override fun getUnsubstitutedInnerClassesScope(): MemberScope = innerClassesScope
 
     private val staticScope = LazyJavaStaticClassScope(c, jClass, this)
@@ -111,13 +125,13 @@ class LazyJavaClassDescriptor(
 
     override fun getCompanionObjectDescriptor(): ClassDescriptor? = null
 
+    override fun getUnsubstitutedMemberScope() = super.getUnsubstitutedMemberScope() as LazyJavaClassMemberScope
     override fun getConstructors() = unsubstitutedMemberScope.constructors()
 
     override val annotations = c.resolveAnnotations(jClass)
 
     private val declaredParameters = c.storageManager.createLazyValue {
-        jClass.typeParameters.map {
-            p ->
+        jClass.typeParameters.map { p ->
             c.typeParameterResolver.resolveTypeParameter(p)
                 ?: throw AssertionError("Parameter $p surely belongs to class $jClass, so it must be resolved")
         }
@@ -125,7 +139,8 @@ class LazyJavaClassDescriptor(
 
     override fun getDeclaredTypeParameters() = declaredParameters()
 
-    override fun getDefaultFunctionTypeForSamInterface(): SimpleType? = c.components.samConversionResolver.resolveFunctionTypeIfSamInterface(this)
+    override fun getDefaultFunctionTypeForSamInterface(): SimpleType? =
+        c.components.samConversionResolver.resolveFunctionTypeIfSamInterface(this)
 
     override fun isDefinitelyNotSamInterface(): Boolean {
         if (kind != ClassKind.INTERFACE) return true
@@ -152,7 +167,7 @@ class LazyJavaClassDescriptor(
     // Checks if any part of compiler has requested scope content
     // It's necessary for IC to figure out if there is a need to track symbols in the class
     fun wasScopeContentRequested() =
-            getUnsubstitutedMemberScope().wasContentRequested() || staticScope.wasContentRequested()
+        getUnsubstitutedMemberScope().wasContentRequested() || staticScope.wasContentRequested()
 
     override fun getSealedSubclasses(): Collection<ClassDescriptor> = emptyList()
 
@@ -190,10 +205,11 @@ class LazyJavaClassDescriptor(
             // Add fake supertype kotlin.collection.Collection<E> to java.util.Collection<E> class if needed
             // Only needed when calculating built-ins member scope
             result.addIfNotNull(
-                    additionalSupertypeClassDescriptor?.let {
-                        createMappedTypeParametersSubstitution(it, this@LazyJavaClassDescriptor)
-                                .buildSubstitutor().substitute(it.defaultType, Variance.INVARIANT)
-                    })
+                additionalSupertypeClassDescriptor?.let {
+                    createMappedTypeParametersSubstitution(it, this@LazyJavaClassDescriptor)
+                        .buildSubstitutor().substitute(it.defaultType, Variance.INVARIANT)
+                }
+            )
 
             result.addIfNotNull(purelyImplementedSupertype)
 
@@ -212,7 +228,7 @@ class LazyJavaClassDescriptor(
             }
 
             val purelyImplementedFqName =
-                    annotatedPurelyImplementedFqName
+                annotatedPurelyImplementedFqName
                     ?: FakePureImplementationsProvider.getPurelyImplementedInterface(fqNameSafe)
                     ?: return null
 
@@ -224,12 +240,10 @@ class LazyJavaClassDescriptor(
 
             val parametersAsTypeProjections = when {
                 typeParameterCount == supertypeParameterCount ->
-                    typeParameters.map {
-                        parameter ->
+                    typeParameters.map { parameter ->
                         TypeProjectionImpl(Variance.INVARIANT, parameter.defaultType)
                     }
-                typeParameterCount == 1 && supertypeParameterCount > 1 && annotatedPurelyImplementedFqName == null ->
-                {
+                typeParameterCount == 1 && supertypeParameterCount > 1 && annotatedPurelyImplementedFqName == null -> {
                     val parameter = TypeProjectionImpl(Variance.INVARIANT, typeParameters.single().defaultType)
                     (1..supertypeParameterCount).map { parameter } // TODO: List(supertypeParameterCount) { parameter }
                 }
@@ -241,7 +255,7 @@ class LazyJavaClassDescriptor(
 
         private fun getPurelyImplementsFqNameFromAnnotation(): FqName? {
             val annotation =
-                    this@LazyJavaClassDescriptor.annotations.findAnnotation(JvmAnnotationNames.PURELY_IMPLEMENTS_ANNOTATION)
+                this@LazyJavaClassDescriptor.annotations.findAnnotation(JvmAnnotationNames.PURELY_IMPLEMENTS_ANNOTATION)
                     ?: return null
 
             val fqNameString = (annotation.allValueArguments.values.singleOrNull() as? StringValue)?.value ?: return null
@@ -262,8 +276,9 @@ class LazyJavaClassDescriptor(
 
     // Only needed when calculating built-ins member scope
     internal fun copy(
-            javaResolverCache: JavaResolverCache, additionalSupertypeClassDescriptor: ClassDescriptor?
-    ) = LazyJavaClassDescriptor(
-            c.replaceComponents(c.components.replace(javaResolverCache = javaResolverCache)),
-            containingDeclaration, jClass, additionalSupertypeClassDescriptor)
+        javaResolverCache: JavaResolverCache, additionalSupertypeClassDescriptor: ClassDescriptor?
+    ): LazyJavaClassDescriptor = LazyJavaClassDescriptor(
+        c.replaceComponents(c.components.replace(javaResolverCache = javaResolverCache)),
+        containingDeclaration, jClass, additionalSupertypeClassDescriptor
+    )
 }
