@@ -23,15 +23,15 @@ import kotlin.reflect.jvm.internal.impl.metadata.jvm.deserialization.JvmBytecode
 import kotlin.reflect.jvm.internal.impl.metadata.jvm.deserialization.JvmMetadataVersion
 
 class KotlinClassHeader(
-        val kind: KotlinClassHeader.Kind,
-        val metadataVersion: JvmMetadataVersion,
-        val bytecodeVersion: JvmBytecodeBinaryVersion,
-        val data: Array<String>?,
-        val incompatibleData: Array<String>?,
-        val strings: Array<String>?,
-        val extraString: String?,
-        val extraInt: Int,
-        val packageName: String?
+    val kind: Kind,
+    val metadataVersion: JvmMetadataVersion,
+    val bytecodeVersion: JvmBytecodeBinaryVersion,
+    val data: Array<String>?,
+    val incompatibleData: Array<String>?,
+    val strings: Array<String>?,
+    private val extraString: String?,
+    val extraInt: Int,
+    val packageName: String?
 ) {
     // See kotlin.Metadata
     enum class Kind(val id: Int) {
@@ -62,15 +62,14 @@ class KotlinClassHeader(
         get() = data.takeIf { kind == Kind.MULTIFILE_CLASS }?.asList().orEmpty()
 
     // TODO: use in incremental compilation
+    @Suppress("unused")
     val multifileClassKind: MultifileClassKind?
         get() = if (kind == Kind.MULTIFILE_CLASS || kind == Kind.MULTIFILE_CLASS_PART) {
             if ((extraInt and JvmAnnotationNames.METADATA_MULTIFILE_PARTS_INHERIT_FLAG) != 0)
                 INHERITING
             else
                 DELEGATING
-        }
-        else
-            null
+        } else null
 
     val isPreRelease: Boolean
         get() = (extraInt and JvmAnnotationNames.METADATA_PRE_RELEASE_FLAG) != 0
