@@ -17,6 +17,7 @@ import kotlin.reflect.jvm.internal.impl.name.ClassId
 import kotlin.reflect.jvm.internal.impl.name.FqName
 import kotlin.reflect.jvm.internal.impl.name.FqNameUnsafe
 import kotlin.reflect.jvm.internal.impl.name.Name
+import kotlin.reflect.jvm.internal.impl.platform.TargetPlatform
 import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils
 import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.getContainingClass
 import kotlin.reflect.jvm.internal.impl.resolve.constants.ConstantValue
@@ -28,6 +29,9 @@ import kotlin.reflect.jvm.internal.impl.types.TypeProjection
 import kotlin.reflect.jvm.internal.impl.types.TypeSubstitutor
 import kotlin.reflect.jvm.internal.impl.types.Variance
 import kotlin.reflect.jvm.internal.impl.types.checker.KotlinTypeChecker
+import kotlin.reflect.jvm.internal.impl.types.checker.KotlinTypeRefiner
+import kotlin.reflect.jvm.internal.impl.types.checker.REFINER_CAPABILITY
+import kotlin.reflect.jvm.internal.impl.types.refinement.TypeRefinement
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.isAnyOrNullableAny
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.makeNullable
 import kotlin.reflect.jvm.internal.impl.utils.DFS
@@ -57,6 +61,9 @@ val DeclarationDescriptor.isExtension: Boolean
 val DeclarationDescriptor.module: ModuleDescriptor
     get() = DescriptorUtils.getContainingModule(this)
 
+val DeclarationDescriptor.platform: TargetPlatform?
+    get() = module.platform
+
 fun ModuleDescriptor.resolveTopLevelClass(topLevelClassFqName: FqName, location: LookupLocation): ClassDescriptor? {
     assert(!topLevelClassFqName.isRoot)
     return getPackage(topLevelClassFqName.parent()).memberScope.getContributedClassifier(
@@ -202,7 +209,9 @@ fun Annotated.getAnnotationRetention(): KotlinRetention? {
     val retentionArgumentValue =
         annotations.findAnnotation(KotlinBuiltIns.FQ_NAMES.retention)?.allValueArguments?.get(RETENTION_PARAMETER_NAME)
                 as? EnumValue ?: return null
-    return KotlinRetention.valueOf(retentionArgumentValue.enumEntryName.asString())
+
+    val retentionArgumentValueName = retentionArgumentValue.enumEntryName.asString()
+    return KotlinRetention.values().firstOrNull { it.name == retentionArgumentValueName }
 }
 
 val Annotated.nonSourceAnnotations: List<AnnotationDescriptor>
@@ -437,3 +446,9 @@ fun DeclarationDescriptor.isAnnotationConstructor(): Boolean =
 
 fun DeclarationDescriptor.isPrimaryConstructorOfInlineClass(): Boolean =
     this is ConstructorDescriptor && this.isPrimary && this.constructedClass.isInline
+
+@TypeRefinement
+fun ModuleDescriptor.getKotlinTypeRefiner(): KotlinTypeRefiner = getCapability(REFINER_CAPABILITY)?.value ?: KotlinTypeRefiner.Default
+
+@UseExperimental(TypeRefinement::class)
+fun ModuleDescriptor.isTypeRefinementEnabled(): Boolean = getCapability(REFINER_CAPABILITY)?.value != null
