@@ -7,22 +7,31 @@ package kotlin.reflect.jvm.internal.impl.types.checker
 
 import kotlin.reflect.jvm.internal.impl.builtins.KotlinBuiltIns
 import kotlin.reflect.jvm.internal.impl.builtins.KotlinBuiltIns.FQ_NAMES
+import kotlin.reflect.jvm.internal.impl.builtins.PrimitiveType
 import kotlin.reflect.jvm.internal.impl.descriptors.*
 import kotlin.reflect.jvm.internal.impl.descriptors.annotations.AnnotationDescriptor
 import kotlin.reflect.jvm.internal.impl.descriptors.annotations.Annotations
+import kotlin.reflect.jvm.internal.impl.name.FqName
+import kotlin.reflect.jvm.internal.impl.name.FqNameUnsafe
+import kotlin.reflect.jvm.internal.impl.name.Name
 import kotlin.reflect.jvm.internal.impl.resolve.calls.inference.CapturedType
 import kotlin.reflect.jvm.internal.impl.resolve.constants.IntegerLiteralTypeConstructor
+import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.fqNameUnsafe
 import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.hasExactAnnotation
 import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.hasNoInferAnnotation
 import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.isExactAnnotation
+import kotlin.reflect.jvm.internal.impl.resolve.substitutedUnderlyingType
 import kotlin.reflect.jvm.internal.impl.types.*
 import kotlin.reflect.jvm.internal.impl.types.model.*
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.asTypeProjection
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.contains
+import kotlin.reflect.jvm.internal.impl.types.typeUtil.representativeUpperBound
 import kotlin.contracts.ExperimentalContracts
 import kotlin.contracts.contract
 
-interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
+interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext, TypeSystemCommonBackendContext {
+    override val isErrorTypeAllowed: Boolean get() = true
+
     override fun TypeConstructorMarker.isDenotable(): Boolean {
         require(this is TypeConstructor, this::errorMessage)
         return this.isDenotable
@@ -49,6 +58,10 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         return this.isError
     }
 
+    override fun TypeConstructorMarker.toErrorType(): SimpleTypeMarker {
+        throw IllegalStateException("Should not be called")
+    }
+
     override fun KotlinTypeMarker.isUninferredParameter(): Boolean {
         require(this is KotlinType, this::errorMessage)
         return ErrorUtils.isUninferredParameter(this)
@@ -145,15 +158,6 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         return this.projectionKind.convertVariance()
     }
 
-
-    private fun TypeVariance.convertVariance(): Variance {
-        return when (this) {
-            TypeVariance.INV -> Variance.INVARIANT
-            TypeVariance.IN -> Variance.IN_VARIANCE
-            TypeVariance.OUT -> Variance.OUT_VARIANCE
-        }
-    }
-
     override fun TypeArgumentMarker.getType(): KotlinTypeMarker {
         require(this is TypeProjection, this::errorMessage)
         return this.type.unwrap()
@@ -214,6 +218,12 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
                 classDescriptor.kind != ClassKind.ANNOTATION_CLASS
     }
 
+    override fun TypeConstructorMarker.isFinalClassOrEnumEntryOrAnnotationClassConstructor(): Boolean {
+        require(this is TypeConstructor, this::errorMessage)
+        val classDescriptor = declarationDescriptor
+        return classDescriptor is ClassDescriptor && classDescriptor.isFinalClass
+    }
+
     override fun SimpleTypeMarker.asArgumentList(): TypeArgumentListMarker {
         require(this is SimpleType, this::errorMessage)
         return this
@@ -331,7 +341,8 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         return builtIns.anyType
     }
 
-    val builtIns: KotlinBuiltIns get() = throw UnsupportedOperationException("Not supported")
+    open val builtIns: KotlinBuiltIns
+        get() = throw UnsupportedOperationException("Not supported")
 
     override fun KotlinTypeMarker.makeDefinitelyNotNullOrNotNull(): KotlinTypeMarker {
         require(this is UnwrappedType, this::errorMessage)
@@ -424,7 +435,7 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
 
     override fun prepareType(type: KotlinTypeMarker): KotlinTypeMarker {
         require(type is UnwrappedType, type::errorMessage)
-        return NewKotlinTypeChecker.transformToNewType(type)
+        return NewKotlinTypeChecker.Default.transformToNewType(type)
     }
 
     override fun DefinitelyNotNullTypeMarker.original(): SimpleTypeMarker {
@@ -445,6 +456,10 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         errorSupportedOnlyInTypeInference()
     }
 
+    override fun createEmptySubstitutor(): TypeSubstitutorMarker {
+        errorSupportedOnlyInTypeInference()
+    }
+
     override fun TypeSubstitutorMarker.safeSubstitute(type: KotlinTypeMarker): KotlinTypeMarker {
         require(type is UnwrappedType, type::errorMessage)
         require(this is TypeSubstitutor, this::errorMessage)
@@ -465,6 +480,10 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         return IntegerLiteralTypeConstructor.findCommonSuperType(explicitSupertypes)
     }
 
+    override fun TypeConstructorMarker.isError(): Boolean {
+        throw IllegalStateException("Should not be called")
+    }
+
     override fun TypeConstructorMarker.getApproximatedIntegerLiteralType(): KotlinTypeMarker {
         require(this is IntegerLiteralTypeConstructor, this::errorMessage)
         return this.getApproximatedType().unwrap()
@@ -488,6 +507,89 @@ interface ClassicTypeSystemContext : TypeSystemInferenceExtensionContext {
         return this is NewCapturedTypeConstructor
     }
 
+    override fun arrayType(componentType: KotlinTypeMarker): SimpleTypeMarker {
+        require(componentType is KotlinType, this::errorMessage)
+        return builtIns.getArrayType(Variance.INVARIANT, componentType)
+    }
+
+    override fun KotlinTypeMarker.isArrayOrNullableArray(): Boolean {
+        require(this is KotlinType, this::errorMessage)
+        return KotlinBuiltIns.isArray(this)
+    }
+
+    override fun KotlinTypeMarker.hasAnnotation(fqName: FqName): Boolean {
+        require(this is KotlinType, this::errorMessage)
+        return annotations.hasAnnotation(fqName)
+    }
+
+    override fun KotlinTypeMarker.getAnnotationFirstArgumentValue(fqName: FqName): Any? {
+        require(this is KotlinType, this::errorMessage)
+        return annotations.findAnnotation(fqName)?.allValueArguments?.values?.firstOrNull()?.value
+    }
+
+    override fun TypeConstructorMarker.getTypeParameterClassifier(): TypeParameterMarker? {
+        require(this is TypeConstructor, this::errorMessage)
+        return declarationDescriptor as? TypeParameterDescriptor
+    }
+
+    override fun TypeConstructorMarker.isInlineClass(): Boolean {
+        require(this is TypeConstructor, this::errorMessage)
+        return (declarationDescriptor as? ClassDescriptor)?.isInline == true
+    }
+
+    override fun TypeParameterMarker.getRepresentativeUpperBound(): KotlinTypeMarker {
+        require(this is TypeParameterDescriptor, this::errorMessage)
+        return representativeUpperBound
+    }
+
+    override fun KotlinTypeMarker.getSubstitutedUnderlyingType(): KotlinTypeMarker? {
+        require(this is KotlinType, this::errorMessage)
+        return substitutedUnderlyingType()
+    }
+
+    override fun TypeConstructorMarker.getPrimitiveType(): PrimitiveType? {
+        require(this is TypeConstructor, this::errorMessage)
+        return KotlinBuiltIns.getPrimitiveType(declarationDescriptor as ClassDescriptor)
+    }
+
+    override fun TypeConstructorMarker.getPrimitiveArrayType(): PrimitiveType? {
+        require(this is TypeConstructor, this::errorMessage)
+        return KotlinBuiltIns.getPrimitiveArrayType(declarationDescriptor as ClassDescriptor)
+    }
+
+    override fun TypeConstructorMarker.isUnderKotlinPackage(): Boolean {
+        require(this is TypeConstructor, this::errorMessage)
+        return declarationDescriptor?.let(KotlinBuiltIns::isUnderKotlinPackage) == true
+    }
+
+    override fun TypeConstructorMarker.getClassFqNameUnsafe(): FqNameUnsafe {
+        require(this is TypeConstructor, this::errorMessage)
+        return (declarationDescriptor as ClassDescriptor).fqNameUnsafe
+    }
+
+    override fun TypeParameterMarker.getName(): Name {
+        require(this is TypeParameterDescriptor, this::errorMessage)
+        return name
+    }
+
+    override fun TypeParameterMarker.isReified(): Boolean {
+        require(this is TypeParameterDescriptor, this::errorMessage)
+        return isReified
+    }
+
+    override fun KotlinTypeMarker.isInterfaceOrAnnotationClass(): Boolean {
+        require(this is KotlinType, this::errorMessage)
+        val descriptor = constructor.declarationDescriptor
+        return descriptor is ClassDescriptor && (descriptor.kind == ClassKind.INTERFACE || descriptor.kind == ClassKind.ANNOTATION_CLASS)
+    }
+}
+
+fun TypeVariance.convertVariance(): Variance {
+    return when (this) {
+        TypeVariance.INV -> Variance.INVARIANT
+        TypeVariance.IN -> Variance.IN_VARIANCE
+        TypeVariance.OUT -> Variance.OUT_VARIANCE
+    }
 }
 
 private fun captureFromExpressionInternal(type: UnwrappedType) = captureFromExpression(type)
@@ -532,7 +634,6 @@ fun Variance.convertVariance(): TypeVariance {
 }
 
 
-@Suppress("EXPERIMENTAL_IS_NOT_ENABLED")
 @UseExperimental(ExperimentalContracts::class)
 fun requireOrDescribe(condition: Boolean, value: Any?) {
     contract {
