@@ -47,16 +47,16 @@ fun findCorrespondingSupertype(
 
             while (currentPathNode != null) {
                 val currentType = currentPathNode.type
-                if (currentType.arguments.any { it.projectionKind != Variance.INVARIANT }) {
-                    substituted = TypeConstructorSubstitution.create(currentType)
-                                        .wrapWithCapturingSubstitution().buildSubstitutor()
-                                        .safeSubstitute(substituted, Variance.INVARIANT)
-                                        .approximate()
+                substituted = if (currentType.arguments.any { it.projectionKind != Variance.INVARIANT }) {
+                    TypeConstructorSubstitution.create(currentType)
+                            .wrapWithCapturingSubstitution().buildSubstitutor()
+                            .safeSubstitute(substituted, Variance.INVARIANT)
+                            .approximate()
                 }
                 else {
-                    substituted = TypeConstructorSubstitution.create(currentType)
-                                        .buildSubstitutor()
-                                        .safeSubstitute(substituted, Variance.INVARIANT)
+                    TypeConstructorSubstitution.create(currentType)
+                            .buildSubstitutor()
+                            .safeSubstitute(substituted, Variance.INVARIANT)
                 }
 
                 isAnyMarkedNullable = isAnyMarkedNullable || currentType.isMarkedNullable
@@ -90,13 +90,15 @@ private fun TypeConstructor.debugInfo() = buildString {
 
     + "type: ${this@debugInfo}"
     + "hashCode: ${this@debugInfo.hashCode()}"
-    + "javaClass: ${this@debugInfo.javaClass.canonicalName}"
+    + "javaClass: ${this@debugInfo::class.java.canonicalName}"
     var declarationDescriptor: DeclarationDescriptor? = declarationDescriptor
     while (declarationDescriptor != null) {
 
         + "fqName: ${DescriptorRenderer.FQ_NAMES_IN_TYPES.render(declarationDescriptor)}"
-        + "javaClass: ${declarationDescriptor.javaClass.canonicalName}"
+        + "javaClass: ${declarationDescriptor::class.java.canonicalName}"
 
         declarationDescriptor = declarationDescriptor.containingDeclaration
     }
 }
+
+interface NewTypeVariableConstructor
